@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 #@+leo-ver=4
 #@+node:@file gmailfs.py
 #@@first
@@ -24,7 +24,7 @@ from errno import *
 from stat import *
 
 import thread
-
+import quopri
 import libgmail
 from constants import *
 
@@ -83,6 +83,8 @@ LinkToTag = 'l'
 
 NumberQueryRetries = 3
 
+regexObjectTrailingMB = re.compile(r'\s?MB$')
+
 def GmailConfig(fname):
     import ConfigParser
     cp = ConfigParser.ConfigParser()
@@ -92,7 +94,8 @@ def GmailConfig(fname):
     try:
       cp.read(fname)
     except:
-      log.error("Unable to read configuaration file: " + fname)
+      log.warning("Unable to read configuration file: " + fname)
+      return
     try:
       options = cp.options("account")
       if "username" in options:
@@ -163,6 +166,15 @@ except:
     pass
 
 
+# This ensures backwards compatability where
+# old filesystems were stored with 7bit encodings
+# but new ones are all quoted printable
+def fixQuotedPrintable(body):
+    fixed = body
+    if re.search("Content-Transfer-Encoding: quoted",body):
+        fixed = quopri.decodestring(body)
+    return fixed
+
 
 def _getMessagesByQuery(ga,queryString):
     tries = 0
@@ -235,6 +247,7 @@ class GmailInode:
             self.msg = msg
             m = re.match(VersionTag+'=(.*) '+RefInodeTag+'=(.*) '+FsNameTag+'='+MagicStartDelim+'(.*)'+MagicEndDelim,msg.subject)
             matchInode = m.group(2)
+
             log.debug("trying to get inode with:"+matchInode+" from subject:"+msg.subject)
             self.inode_msg = self.getinodeMsg(matchInode)
             self.setInode()
@@ -259,7 +272,7 @@ class GmailInode:
         if not thread._messages:
           thread._messages = thread._getMessages(thread)
 
-        return thread._messages[0]
+        return thread._messages[len(thread._messages)-1]
     #@-node:getinodeMsg
     
     def update(self):
@@ -284,8 +297,9 @@ class GmailInode:
         gmsg = libgmail.GmailComposedMessage(username, subject, body)
         if _sendMessage(self.ga,gmsg):
             log.debug("Sent "+subject+" ok")
-            log.debug("trashing old inode:"+str(self.inode_msg.subject))
-            self.ga.trashMessage(self.inode_msg)
+            if (self.inode_msg):
+                log.debug("trashing old inode:"+str(self.inode_msg.subject))
+                self.ga.trashMessage(self.inode_msg)
             self.inode_msg = self.getinodeMsg(str(self.ino))
         else:
             e = OSError("Couldnt send mesg:"+gmsg.subject)
@@ -301,6 +315,7 @@ class GmailInode:
         try:
             subject = self.inode_msg.subject
             body = self.inode_msg.source
+	    body = fixQuotedPrintable(body)
             log.debug("setting inode from subject:"+subject)
             log.debug("and body:"+body)
             m = re.match(VersionTag+'=(.*) '+InodeTag+'=(.*) '+DevTag+'=(.*) '+NumberLinksTag+'=(.*) '+FsNameTag+'='+MagicStartDelim+'(.*)'+
@@ -309,10 +324,16 @@ class GmailInode:
             self.ino = int(m.group(2))
             self.dev = int(m.group(3))
             self.nlink = int(m.group(4))
-            m = re.search(ModeTag+'=(.*) '+UidTag+'=(.*) '+GidTag+'=(.*) '+
-                          SizeTag+'=(.*) '+AtimeTag+'=(.*) '+MtimeTag+'=(.*) '+
-                          CtimeTag+'=(.*) '+BlockSizeTag+'=(.*)', body)
-            self.mode = int(m.group(1))
+            #quotedEquals = "=(?:3D)?(.*)"
+            quotedEquals = "=(.*)"
+            m = re.search(ModeTag+quotedEquals+' '+UidTag+quotedEquals+' '+
+  			  GidTag+quotedEquals+' '+
+                          SizeTag+quotedEquals+' '+
+			  AtimeTag+quotedEquals+' '+
+			  MtimeTag+quotedEquals+' '+
+                          CtimeTag+quotedEquals+' '+
+			  BlockSizeTag+quotedEquals, body)
+	    self.mode = int(m.group(1))
             self.uid = int(m.group(2))
             self.gid = int(m.group(3))
             self.size = int(m.group(4))
@@ -495,7 +516,7 @@ class OpenFile:
         thread = folder._threads[0]
         if not thread._messages:
             thread._messages = thread._getMessages(thread)
-        msg = thread._messages[0]
+        msg = thread._messages[len(thread._messages)-1]
         log.debug("got msg with subject:"+msg.subject)
         log.debug("got "+str(len(msg.attachments))+" attachments")
         a = msg.attachments[0]
@@ -523,7 +544,7 @@ class Gmailfs(Fuse):
     
         Fuse.__init__(self, *args, **kw)
     
-        if 0:
+        if 1:
             print "gmailfs.py:Gmailfs:mountpoint: %s" % repr(self.mountpoint)
             print "gmailfs.py:Gmailfs:unnamed mount options: %s" % self.optlist
             print "gmailfs.py:Gmailfs:named mount options: %s" % self.optdict
@@ -653,8 +674,9 @@ class Gmailfs(Fuse):
             e.errno = EINVAL
             raise e
         log.debug("about to follow link in body:"+inode.msg.source)
+	body = fixQuotedPrintable(inode.msg.source)
         m = re.search(LinkToTag+'='+LinkStartDelim+'(.*)'+
-                      LinkEndDelim,inode.msg.source)
+                      LinkEndDelim,body)
         return m.group(1)
     #@-node:readlink
     
@@ -687,8 +709,9 @@ class Gmailfs(Fuse):
                  else:
                      # Filename was too long, have to fetch message.
                      log.debug("Long filename, had to fetch message")
+		     body = fixQuotedPrintable(msg.source)
                      m = re.search(FileNameTag+'='+FileStartDelim+'(.*)'+
-                                   FileEndDelim, msg.source)
+                                   FileEndDelim, body)
                      filename = m.group(1)
 
                  log.debug("Found file:"+filename)
@@ -781,10 +804,11 @@ class Gmailfs(Fuse):
         subject = ( VersionTag+"="+GMAILFS_VERSION+
                   " "+RefInodeTag+"="+m.group(2)+
 	          " "+FsNameTag+"="+MagicStartDelim+ fsNameVar +MagicEndDelim )
+	bodytmp = fixQuotedPrintable(msg.source)
         m = re.search(FileNameTag+'='+FileStartDelim+'(.*)'+FileEndDelim+
 	             ' '+PathNameTag+'='+PathStartDelim+'(.*)'+PathEndDelim+
 		     ' '+LinkToTag+'='+LinkStartDelim+'(.*)'+LinkEndDelim,
-                      msg.source)
+                      bodytmp)
         body = (FileNameTag+"="+FileStartDelim+ name +FileEndDelim+
 	       " "+PathNameTag+"="+PathStartDelim+ fspath +PathEndDelim+
 	       " "+LinkToTag+"="+LinkStartDelim+ m.group(3) +LinkEndDelim )
@@ -1004,25 +1028,27 @@ class Gmailfs(Fuse):
             - blocksize - size of file blocks, in bytes
             - totalblocks - total number of blocks in the filesystem
             - freeblocks - number of free blocks
+            - availblocks - number of blocks available to non-superuser
             - totalfiles - total number of file inodes
             - freefiles - nunber of free file inodes
     
         Feel free to set any of the above values to 0, which tells
         the kernel that the info is not available.
         """
-       
-        blocks_size = 1024
-        blocks = 1000*1024
+        block_size = 1024
         quotaInfo = self.ga.getQuotaInfo()
         quotaMbUsed = quotaInfo[QU_SPACEUSED]
         quotaMbTotal = quotaInfo[QU_QUOTA]
+        blocks = int(regexObjectTrailingMB.sub('', quotaMbTotal)) * 1024 * 1024 / block_size
         quotaPercent = quotaInfo[QU_PERCENT]
+        blocks_free = blocks - int(regexObjectTrailingMB.sub('', quotaMbUsed)) * 1024 * 1024 / block_size
+        blocks_avail = blocks_free # I guess...
         log.debug("%s of %s used. (%s)\n" % (quotaMbUsed, quotaMbTotal, quotaPercent))
-        blocks_free = blocks-(int(quotaMbUsed[:-3])*1024)
+        log.debug("Blocks: %s free, %s total\n" % (blocks_free, blocks))
         files = 0
         files_free = 0
         namelen = 80
-        return (blocks_size, blocks, blocks_free, files, files_free, namelen)
+        return (block_size, blocks, blocks_free, blocks_avail, files, files_free, namelen)
     #@-node:statfs
 
     #@+node:fsync
@@ -1036,6 +1062,7 @@ class Gmailfs(Fuse):
     #@-node:fsync
 
 
+
     def getinodemsg(self, path):
       try:
           log.debug("check getnodemsg:"+path)
@@ -1051,6 +1078,7 @@ class Gmailfs(Fuse):
                                        FileStartDelim+name+FileEndDelim+
                                        ' '+PathNameTag+'='+PathStartDelim+
                                        fspath +PathEndDelim)
+				       
           if len(folder)!=1:
               return None;
           if len(folder._threads[0])!=1:
@@ -1058,7 +1086,7 @@ class Gmailfs(Fuse):
           thread = folder._threads[0]
           if not thread._messages:
             thread._messages = thread._getMessages(thread)
-          return thread._messages[0]
+          return thread._messages[len(thread._messages)-1]
       except:
           traceback.print_exc(file=sys.stdout)
           log.error("no slash in path:"+path)
@@ -1084,7 +1112,6 @@ class Gmailfs(Fuse):
     
 #@-node:class Gmailfs
 #@+node:mainline
-import profile
 
 if __name__ == '__main__':
 	server = Gmailfs()
